/* This file is automatically generated by configure.  Do not modify by hand. */
netsnmp_udpipv6_ctor();
netsnmp_tcpipv6_ctor();
netsnmp_udp_ctor();
netsnmp_tcp_ctor();
netsnmp_alias_ctor();
netsnmp_unix_ctor();
