/* This file is automatically generated by configure.  Do not modify by hand. */
  if (should_init("snmpEngine")) init_snmpEngine();
  if (should_init("snmpMPDStats")) init_snmpMPDStats();
  if (should_init("usmStats")) init_usmStats();
  if (should_init("usmUser")) init_usmUser();
  if (should_init("system_mib")) init_system_mib();
  if (should_init("sysORTable")) init_sysORTable();
  if (should_init("snmp_mib")) init_snmp_mib();
  if (should_init("vacm_vars")) init_vacm_vars();
  if (should_init("setSerialNo")) init_setSerialNo();
  if (should_init("at")) init_at();
  if (should_init("ip")) init_ip();
  if (should_init("tcp")) init_tcp();
  if (should_init("udp")) init_udp();
  if (should_init("ipv6")) init_ipv6();
  if (should_init("icmp")) init_icmp();
  if (should_init("proc")) init_proc();
  if (should_init("versioninfo")) init_versioninfo();
  if (should_init("pass")) init_pass();
  if (should_init("pass_persist")) init_pass_persist();
  if (should_init("disk_hw")) init_disk_hw();
  if (should_init("loadave")) init_loadave();
  if (should_init("extend")) init_extend();
  if (should_init("errormib")) init_errormib();
  if (should_init("file")) init_file();
  if (should_init("dlmod")) init_dlmod();
  if (should_init("proxy")) init_proxy();
  if (should_init("logmatch")) init_logmatch();
  if (should_init("memory")) init_memory();
  if (should_init("vmstat")) init_vmstat();
  if (should_init("snmpNotifyTable")) init_snmpNotifyTable();
  if (should_init("snmpNotifyFilterProfileTable")) init_snmpNotifyFilterProfileTable();
  if (should_init("notification_log")) init_notification_log();
  if (should_init("snmpTargetAddrEntry")) init_snmpTargetAddrEntry();
  if (should_init("snmpTargetParamsEntry")) init_snmpTargetParamsEntry();
  if (should_init("target_counters")) init_target_counters();
  if (should_init("nsTransactionTable")) init_nsTransactionTable();
  if (should_init("nsModuleTable")) init_nsModuleTable();
  if (should_init("nsDebug")) init_nsDebug();
  if (should_init("nsCache")) init_nsCache();
  if (should_init("nsLogging")) init_nsLogging();
  if (should_init("nsVacmAccessTable")) init_nsVacmAccessTable();
  if (should_init("mteScalars")) init_mteScalars();
  if (should_init("mteTrigger")) init_mteTrigger();
  if (should_init("mteTriggerTable")) init_mteTriggerTable();
  if (should_init("mteTriggerDeltaTable")) init_mteTriggerDeltaTable();
  if (should_init("mteTriggerExistenceTable")) init_mteTriggerExistenceTable();
  if (should_init("mteTriggerBooleanTable")) init_mteTriggerBooleanTable();
  if (should_init("mteTriggerThresholdTable")) init_mteTriggerThresholdTable();
  if (should_init("mteTriggerConf")) init_mteTriggerConf();
  if (should_init("mteEvent")) init_mteEvent();
  if (should_init("mteEventTable")) init_mteEventTable();
  if (should_init("mteEventSetTable")) init_mteEventSetTable();
  if (should_init("mteEventNotificationTable")) init_mteEventNotificationTable();
  if (should_init("mteEventConf")) init_mteEventConf();
  if (should_init("mteObjects")) init_mteObjects();
  if (should_init("mteObjectsTable")) init_mteObjectsTable();
  if (should_init("mteObjectsConf")) init_mteObjectsConf();
  if (should_init("schedCore")) init_schedCore();
  if (should_init("schedConf")) init_schedConf();
  if (should_init("schedTable")) init_schedTable();
  if (should_init("override")) init_override();
  if (should_init("hrh_storage")) init_hrh_storage();
  if (should_init("hrh_filesys")) init_hrh_filesys();
  if (should_init("hrSWInstalledTable")) init_hrSWInstalledTable();
  if (should_init("hrSWRunTable")) init_hrSWRunTable();
  if (should_init("hr_system")) init_hr_system();
  if (should_init("hr_device")) init_hr_device();
  if (should_init("hr_other")) init_hr_other();
  if (should_init("hr_proc")) init_hr_proc();
  if (should_init("hr_network")) init_hr_network();
  if (should_init("hr_print")) init_hr_print();
  if (should_init("hr_disk")) init_hr_disk();
  if (should_init("hr_partition")) init_hr_partition();
  if (should_init("vacm_context")) init_vacm_context();
  if (should_init("var_route")) init_var_route();
  if (should_init("tcpTable")) init_tcpTable();
  if (should_init("udpTable")) init_udpTable();
  if (should_init("ip_scalars")) init_ip_scalars();
  if (should_init("snmpNotifyTable_data")) init_snmpNotifyTable_data();
  if (should_init("snmpNotifyFilterTable_data_storage")) init_snmpNotifyFilterTable_data_storage();
  if (should_init("snmpNotifyFilterTable")) init_snmpNotifyFilterTable();
  if (should_init("snmpNotifyFilterProfileTable_data")) init_snmpNotifyFilterProfileTable_data();
  if (should_init("snmpTargetAddrEntry_data")) init_snmpTargetAddrEntry_data();
  if (should_init("snmpTargetParamsEntry_data")) init_snmpTargetParamsEntry_data();
  if (should_init("swinst")) init_swinst();
  if (should_init("swrun")) init_swrun();
  if (should_init("hrSWRunPerfTable")) init_hrSWRunPerfTable();
  if (should_init("ifTable")) init_ifTable();
  if (should_init("ifXTable")) init_ifXTable();
  if (should_init("ipAddressTable")) init_ipAddressTable();
  if (should_init("ipAddressPrefixTable")) init_ipAddressPrefixTable();
  if (should_init("ipDefaultRouterTable")) init_ipDefaultRouterTable();
  if (should_init("inetNetToMediaTable")) init_inetNetToMediaTable();
  if (should_init("ipSystemStatsTable")) init_ipSystemStatsTable();
  if (should_init("ipv6ScopeZoneIndexTable")) init_ipv6ScopeZoneIndexTable();
  if (should_init("ipIfStatsTable")) init_ipIfStatsTable();
  if (should_init("ipCidrRouteTable")) init_ipCidrRouteTable();
  if (should_init("inetCidrRouteTable")) init_inetCidrRouteTable();
  if (should_init("tcpConnectionTable")) init_tcpConnectionTable();
  if (should_init("tcpListenerTable")) init_tcpListenerTable();
  if (should_init("udpEndpointTable")) init_udpEndpointTable();
  if (should_init("hw_fsys")) init_hw_fsys();
  if (should_init("hw_mem")) init_hw_mem();
  if (should_init("cpu")) init_cpu();
  if (should_init("cpu_linux")) init_cpu_linux();
  if (should_init("interface")) init_interface();
